/*
	LUSScreenSaver Plugin LUSSSPicture ( SlideShowModule )
	Copyright (C) 2003 ONO,Masayuki.<mono101@skill-engineering.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "LUSSSMessageSetting.h"
#include <qfile.h>
#include <qpushbutton.h>
#include <qpixmap.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qwhatsthis.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qfontdatabase.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qgroupbox.h>
#include <qpe/config.h>
#include <qpe/qpeapplication.h>
#include <sl/slcolorselector.h>

#include <qtextcodec.h>
#define _T(str) QTextCodec::codecForLocale()->toUnicode(str)

LUSSSMessageSetting::LUSSSMessageSetting( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
	if ( ! name )
	{
		setName( "SettingDialog" );
	}

	setCaption( "SETTINGS" );

	BaseLayout = new QVBoxLayout( this ); 
	BaseLayout->setSpacing( 6 );
	BaseLayout->setMargin( 11 );

	BaseGroupBox = new QGroupBox( this, "BaseGroupBox" );
	BaseGroupBox->setTitle( "" );
	BaseGroupBox->setColumnLayout(0, Qt::Vertical );
	BaseGroupBox->layout()->setSpacing( 0 );
	BaseGroupBox->layout()->setMargin( 0 );
	BaseGroupBoxLayout = new QVBoxLayout( BaseGroupBox->layout() );
	BaseGroupBoxLayout->setAlignment( Qt::AlignTop );
	BaseGroupBoxLayout->setSpacing( 6 );
	BaseGroupBoxLayout->setMargin( 11 );

	ColorButtonsLayout = new QHBoxLayout( this );
	ColorButtonsLayout->setSpacing( 6 );
	ColorButtonsLayout->setMargin( 11 );

	FontSetsLayout = new QHBoxLayout( this );
	FontSetsLayout->setSpacing( 6 );
	FontSetsLayout->setMargin( 11 );

	ChecksLayout = new QHBoxLayout( this );
	ChecksLayout->setSpacing( 6 );
	ChecksLayout->setMargin( 11 );

	IntervalLayout = new QHBoxLayout( this );
	IntervalLayout->setSpacing( 6 );
	IntervalLayout->setMargin( 11 );

	DefButtonsLayout = new QHBoxLayout( this );
	DefButtonsLayout->setSpacing( 6 );
	DefButtonsLayout->setMargin( 11 );

	Config cfg( "LUSScreenSaver" );
	cfg.setGroup( "LUSSSMessage" );

	MessageLabel = new QLabel( _T( "ɽå" ), BaseGroupBox, "MessageLabel" );
	
	MessageEdit = new QLineEdit( BaseGroupBox, "MessageEdit" );
	MessageEdit->setText( cfg.readEntry( "Message", "LUSSSMessage" ) );

	IntervalLabel = new QLabel( _T( "ٱ" ), BaseGroupBox, "IntervalLabel" );

	IntervalSpin = new QSpinBox( 1, 600, 1, BaseGroupBox, "IntervalSpin" );
	IntervalSpin->setSuffix( _T( " ߥ" ) );
	IntervalSpin->setValue( cfg.readNumEntry( "Interval", 10 ) );

	IntervalLayout->addWidget( IntervalLabel );
	IntervalLayout->addWidget( IntervalSpin );

	QFontMetrics fm( font() );
	FontColorLabel = new QLabel( _T( "ʸ" ), BaseGroupBox, "FontColorLabel" );
	FontColorButton = new QPushButton( this, "FontColorButton" );
	FontColorButton->setText( _T( "" ) );
	QColor colorPen( cfg.readNumEntry( "FR", 255 ),
						cfg.readNumEntry( "FG", 255 ),
						cfg.readNumEntry( "FB", 255 ) );
	QPixmap tmpPixmap( fm.width( "    " ), fm.height() * 2 );
	tmpPixmap.fill( colorPen );
	FontColorDisp = new QLabel( "", BaseGroupBox, "FontColorDisp" );
	FontColorDisp->setPixmap( tmpPixmap );

	BackgroundColorLabel = new QLabel( _T( "طʿ" ), BaseGroupBox, "BackgroundColorLabel" );
	BackgroundColorButton = new QPushButton( this, "BackgroundColorButton" );
	BackgroundColorButton->setText( _T( "" ) );
	QColor colorBack( cfg.readNumEntry( "BR", 0 ),
						cfg.readNumEntry( "BG", 0 ),
						cfg.readNumEntry( "BB", 0 ) );
	tmpPixmap.fill( colorBack );
	BackgroundColorDisp = new QLabel( "", BaseGroupBox, "BackgroundColorDisp" );
	BackgroundColorDisp->setPixmap( tmpPixmap );

	ColorButtonsLayout->addWidget( FontColorLabel );
	ColorButtonsLayout->addWidget( FontColorDisp );
	ColorButtonsLayout->addWidget( FontColorButton );
	ColorButtonsLayout->addWidget( BackgroundColorLabel );
	ColorButtonsLayout->addWidget( BackgroundColorDisp );
	ColorButtonsLayout->addWidget( BackgroundColorButton );

	QFontDatabase fdb;
	FontFamilyCombo = new QComboBox( BaseGroupBox, "FontFamilyCombo" );
	QStringList fonts = fdb.families();
	FontFamilyCombo->insertStringList( fonts );
	QString font_name = cfg.readEntry( "FontFamily", "lcfont" );
	FontFamilyCombo->setCurrentItem( 0 );
	int nFontsCount = FontFamilyCombo->maxCount();
	for ( int nCnt = 0; nCnt < nFontsCount; nCnt ++ )
	{
		if ( FontFamilyCombo->text( nCnt ) == font_name )
		{
			FontFamilyCombo->setCurrentItem( nCnt );
			break;
		}
	}

	FontPointSizeCombo = new QComboBox( BaseGroupBox, "FontPointSizeCombo" );
	QValueList<int> fontSizes = fdb.pointSizes( FontFamilyCombo->currentText() );
	int nFontSizesCount = fontSizes.count();
	for ( int nCnt = 0; nCnt < nFontSizesCount; nCnt ++ )
	{
		FontPointSizeCombo->insertItem( QString( "%1" ).arg( fontSizes[ nCnt ] ) );
	}
	QString sDefSize = QString( "%1" ).arg( cfg.readNumEntry( "FontSize" ) );
	FontPointSizeCombo->setCurrentItem( 0 );
	for ( int nCnt = 0; nCnt < nFontSizesCount; nCnt ++ )
	{
		if ( FontPointSizeCombo->text( nCnt ) == sDefSize )
		{
			FontPointSizeCombo->setCurrentItem( nCnt );
			break;
		}
	}

	FontSetsLayout->addWidget( FontFamilyCombo );
	FontSetsLayout->addWidget( FontPointSizeCombo );

	ItalicCheck = new QCheckBox( _T( "" ), BaseGroupBox, "ItalicCheck" );
	ItalicCheck->setChecked( cfg.readBoolEntry( "FontItalic", FALSE ) );

	UnderlineCheck = new QCheckBox( _T(""), BaseGroupBox, "UnderlineCheck" );
	UnderlineCheck->setChecked( cfg.readBoolEntry( "FontUnderline", FALSE ) );
	
	StrikeOutCheck = new QCheckBox( _T(""), BaseGroupBox, "StrikeOutCheck" );
	StrikeOutCheck->setChecked( cfg.readBoolEntry( "FontStrikeOut", FALSE ) );

	ChecksLayout->addWidget( ItalicCheck );
	ChecksLayout->addWidget( UnderlineCheck );
	ChecksLayout->addWidget( StrikeOutCheck );
	
	PushButton1 = new QPushButton( this, "PushButton1" );
	PushButton1->setText( _T( "Ŭ" ) );

	PushButton2 = new QPushButton( this, "PushButton2" );
	PushButton2->setText( _T( "󥻥" ) );
	
	DefButtonsLayout->addWidget( PushButton1 );
	DefButtonsLayout->addWidget( PushButton2 );

	BaseGroupBoxLayout->addWidget( MessageLabel );
	BaseGroupBoxLayout->addWidget( MessageEdit );
	BaseGroupBoxLayout->addLayout( IntervalLayout );
	BaseGroupBoxLayout->addLayout( ColorButtonsLayout );
	BaseGroupBoxLayout->addLayout( FontSetsLayout );
	BaseGroupBoxLayout->addLayout( ChecksLayout );
	BaseLayout->addWidget( BaseGroupBox );
	BaseLayout->addLayout( DefButtonsLayout );
	
	connect( FontFamilyCombo, SIGNAL( activated( const QString& ) ), this, SLOT( changedFontFamily( const QString& ) ) );
	connect( FontColorButton, SIGNAL( clicked() ), this, SLOT( selectFontColor() ) );
	connect( BackgroundColorButton, SIGNAL( clicked() ), this, SLOT( selectBackgroundColor() ) );
	connect( PushButton1 ,SIGNAL( clicked() ), this, SLOT( accept() ) );
	connect( PushButton2 ,SIGNAL( clicked() ), this, SLOT( close() ) );

	showMaximized();
}

void LUSSSMessageSetting::changedFontFamily( const QString& name )
{
	Config cfg( "LUSScreenSaver" );
	cfg.setGroup( "LUSSSMessage" );
	QFontDatabase fdb;
	FontPointSizeCombo->clear();
	QValueList<int> fontSizes = fdb.pointSizes( name );
	int nFontSizesCount = fontSizes.count();
	for ( int nCnt = 0; nCnt < nFontSizesCount; nCnt ++ )
	{
		FontPointSizeCombo->insertItem( QString( "%1" ).arg( fontSizes[ nCnt ] ) );
	}
	QString sDefSize = QString( "%1" ).arg( cfg.readNumEntry( "FontSize" ) );
	FontPointSizeCombo->setCurrentItem( 0 );
	for ( int nCnt = 0; nCnt < nFontSizesCount; nCnt ++ )
	{
		if ( FontPointSizeCombo->text( nCnt ) == sDefSize )
		{
			FontPointSizeCombo->setCurrentItem( nCnt );
			break;
		}
	}
}

void LUSSSMessageSetting::selectFontColor()
{
	SlColorSelectorPopup* colorSelector = new SlColorSelectorPopup( FALSE, this );
	connect( colorSelector, SIGNAL( selected( QRgb ) ), this, SLOT( selectedFontColor( QRgb ) ) );
	QPoint tPoint( FontColorButton->geometry().left(), FontColorButton->geometry().top() );
	tPoint = mapToGlobal( tPoint );
	colorSelector->popup( tPoint );
}

void LUSSSMessageSetting::selectedFontColor( QRgb rgb )
{
	QFontMetrics fm( font() );
	QPixmap tmpPixmap( fm.width( "    " ), fm.height() * 2 );
	tmpPixmap.fill( rgb );
	FontColorDisp->setPixmap( tmpPixmap );
}

void LUSSSMessageSetting::selectBackgroundColor()
{
	SlColorSelectorPopup* colorSelector = new SlColorSelectorPopup( FALSE, this );
	connect( colorSelector, SIGNAL( selected( QRgb ) ), this, SLOT( selectedBackgroundColor( QRgb ) ) );
	QPoint tPoint( BackgroundColorButton->geometry().left(), BackgroundColorButton->geometry().top() );
	tPoint = mapToGlobal( tPoint );
	colorSelector->popup( tPoint );
}

void LUSSSMessageSetting::selectedBackgroundColor( QRgb rgb )
{
	QFontMetrics fm( font() );
	QPixmap tmpPixmap( fm.width( "    " ), fm.height() * 2 );
	tmpPixmap.fill( rgb );
	BackgroundColorDisp->setPixmap( tmpPixmap );
}

void LUSSSMessageSetting::accept()
{
	Config cfg( "LUSScreenSaver" );
	cfg.setGroup( "LUSSSMessage" );

	if ( ! MessageEdit->text().isEmpty() )
	{
		cfg.writeEntry( "Message", MessageEdit->text() );
	}
	else
	{
		cfg.writeEntry( "Message", QString( "LUSSSMessage" ) );
	}

	QImage fontColorImage = FontColorDisp->pixmap()->convertToImage();
	QImage backgroundColorImage = BackgroundColorDisp->pixmap()->convertToImage();

	QRgb fontColor = fontColorImage.pixel(1,1);
	QRgb backColor = backgroundColorImage.pixel(1,1);

	cfg.writeEntry( "FR", qRed( fontColor ) );
	cfg.writeEntry( "FG", qGreen( fontColor ) );
	cfg.writeEntry( "FB", qBlue( fontColor ) );
	
	cfg.writeEntry( "BR", qRed( backColor ) );
	cfg.writeEntry( "BG", qGreen( backColor ) );
	cfg.writeEntry( "BB", qBlue( backColor ) );

	cfg.writeEntry( "Interval", IntervalSpin->value() );
	cfg.writeEntry( "FontFamily", FontFamilyCombo->currentText() );
	cfg.writeEntry( "FontSize", FontPointSizeCombo->currentText() );
	cfg.writeEntry( "FontItalic", ItalicCheck->isChecked() );
	cfg.writeEntry( "FontUnderline", UnderlineCheck->isChecked() );
	cfg.writeEntry( "FontStrikeOut", StrikeOutCheck->isChecked() );
	close();
}

void LUSSSMessageSetting::closeEvent( QCloseEvent* e )
{
	e->accept();
}


LUSSSMessageSetting::~LUSSSMessageSetting()
{
    // no need to delete child widgets, Qt does it all for us
}

