/*
	LUSScreenSaver Plugin LUSSSMessage ( Ticking Message )
	Copyright (C) 2003 ONO,Masayuki.<mono101@skill-engineering.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "LUSSSMessageWidget.h"

#include <qtextcodec.h>
#define _T(str) QTextCodec::codecForLocale()->toUnicode(str)

LUSSSMessageWidget::LUSSSMessageWidget( QWidget* parent, const char* name )
	: QWidget( parent, name, WDestructiveClose )
{
	clearWFlags( WStyle_Title );
}

LUSSSMessageWidget::~LUSSSMessageWidget(  )
{
}

void LUSSSMessageWidget::Start()
{
	Config cfg( "LUSScreenSaver" );
	cfg.setGroup( "LUSSSMessage" );
	QString sMessage = cfg.readEntry( "Message", "LUSSSMessage" );
	QColor colorBack( cfg.readNumEntry( "BR", 0 ),
						cfg.readNumEntry( "BG", 0 ),
						cfg.readNumEntry( "BB", 0 ) );
	QColor colorPen( cfg.readNumEntry( "FR", 255 ),
						cfg.readNumEntry( "FG", 255 ),
						cfg.readNumEntry( "FB", 255 ) );
	setBackgroundColor( colorBack );
	QFont oFont( cfg.readEntry( "FontFamily", font().family() ),
					font().pointSize(),
					QFont::Normal,
					cfg.readBoolEntry( "FontItalic", font().italic() ) );
	int nFontPointSize = cfg.readNumEntry( "FontSize", font().pointSize() );
	bool bUnderline = cfg.readBoolEntry( "FontUnderline", font().underline() );
	bool bStrikeOut = cfg.readBoolEntry( "FontStrikeOut", font().strikeOut() );
	
	QFontMetrics fm( oFont );
	QPixmap tmpPixmap;
	tmpPixmap.resize( QSize( fm.width( sMessage ), fm.height() ) );
	tmpPixmap.fill( colorBack );
	QPainter p( &tmpPixmap );
	p.setPen( colorPen );
	p.drawText( QPoint( 0, fm.height() ), sMessage );
	if ( bUnderline )
	{
		p.drawLine( QPoint( 0, fm.height() - 1 ), QPoint( fm.width( sMessage ), fm.height() - 1 ) );
	}
	if ( bStrikeOut )
	{
		p.drawLine( QPoint( 0, fm.height() / 2 ), QPoint( fm.width( sMessage ), fm.height() / 2 ) );
	}
	QImage tmpImage = tmpPixmap.convertToImage();
	double fScale = nFontPointSize / font().pointSize();
	fScale = sqrt( fScale );
	tmpImage = tmpImage.smoothScale( tmpImage.size().width() * fScale, tmpImage.size().height() * fScale );
	m_oPixmap.convertFromImage( tmpImage );
	
	m_oPoint = QPoint( width(), rand() / ( RAND_MAX / ( height() - m_oPixmap.height() ) + 1 ) );
	erase();

	show();

	repaint( TRUE );

	int nInterval = cfg.readNumEntry( "Interval", 10 );
	timer = new QTimer(this);
	connect( timer, SIGNAL( timeout() ), this, SLOT( timerDo() ) );
	timer->start( nInterval, false );
}

void LUSSSMessageWidget::Stop()
{
	if ( !timer )
	{
		timer->stop();
		delete timer;
	}
}

void LUSSSMessageWidget::timerDo()
{
	QPainter painter;
	painter.begin( this );
	painter.drawPixmap( m_oPoint, m_oPixmap );
	painter.setPen( backgroundColor() );
	painter.drawLine( m_oPoint + QPoint( m_oPixmap.width() + 1, 0 ), m_oPoint + QPoint( m_oPixmap.width() + 1, m_oPixmap.height() ) );

	m_oPoint -= QPoint( 1, 0 );

	if ( m_oPoint.x() == m_oPixmap.width() * -1 )
	{
		painter.drawLine( m_oPoint + QPoint( m_oPixmap.width() + 1, 0 ), m_oPoint + QPoint( m_oPixmap.width() + 1, m_oPixmap.height() ) );
		m_oPoint -= QPoint( 1, 0 );
		painter.drawLine( m_oPoint + QPoint( m_oPixmap.width() + 1, 0 ), m_oPoint + QPoint( m_oPixmap.width() + 1, m_oPixmap.height() ) );
		m_oPoint = QPoint( width(), rand() / ( RAND_MAX / ( height() - m_oPixmap.height() ) + 1 ) );
	}
	painter.end();
}

void LUSSSMessageWidget::paintEvent( QPaintEvent * )
{
}

